import time
import logging

from watchdog.observers import Observer

from .constants import (
    ENV,
    TOKEN,
    MESSAGE_TIMEOUT,
    MAIL_PATH,
    MAIL_FOLDER,
    MAIL_FILE,
    ATTACHMENTS_FOLDER
)
from .telegram_bot import TelegramBot
from .email import EmailHandler
from .utils import clear_attachments

def main():
    if ENV != "production":
        logging.basicConfig(level=logging.DEBUG)
    logging.info("Entering main function")
    bot = TelegramBot(
        TOKEN,
        MESSAGE_TIMEOUT,
    )

    # Set handler and observer then start
    event_handler = EmailHandler(
        bot, str((MAIL_FOLDER / (MAIL_FILE + ".lock"))), MAIL_PATH
    )
    observer = Observer()
    observer.schedule(event_handler, path=f"{MAIL_FOLDER}/", recursive=True)
    observer.start()

    # Consume once on startup
    event_handler.consume_emails()
    try:
        while True:
            clear_attachments(ATTACHMENTS_FOLDER)
            time.sleep(86400)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()


if __name__ == "__main__":
    main()



import sys
import pathlib
import logging
import configparser

# Required variables
MAIL_FOLDER = MAIL_FILE = MAIL_PATH = TOKEN = TRANSFERS = None

# Optional variables
ENV = "production"
CONFIG_PATHS = ["/etc/email-to-telegram/config.ini", "config.ini"]
ATTACHMENTS_FOLDER = "/tmp/telegram-attachments_folder"
MESSAGE_TIMEOUT = 3
DISABLE_TEXT = False
DISABLE_PHOTO = False
DISABLE_CAPTION = False


def load_config(config_path):
    global MAIL_FOLDER, MAIL_FILE, MAIL_PATH, ATTACHMENTS_FOLDER, MESSAGE_TIMEOUT, TOKEN, ENV, TRANSFERS, DISABLE_TEXT, DISABLE_PHOTO, DISABLE_CAPTION

    config = configparser.ConfigParser()
    config.read(config_path)

    # Required variables
    TOKEN = config["DEFAULT"]["token"]
    MAIL_PATH = pathlib.Path((config["DEFAULT"]["mail_path"]))
    MAIL_FOLDER = MAIL_PATH.parent
    MAIL_FILE = MAIL_PATH.name

    # Optional variables
    MESSAGE_TIMEOUT = config["DEFAULT"].getint("message_timeout", MESSAGE_TIMEOUT)
 
    ATTACHMENTS_FOLDER = config["DEFAULT"].get(
        "attachments_folder", ATTACHMENTS_FOLDER
    )
    ENV = config["DEFAULT"].get("env", ENV)
    DISABLE_TEXT = config["DEFAULT"].getboolean("disable_text", DISABLE_TEXT)
    DISABLE_PHOTO = config["DEFAULT"].getboolean("disable_photo", DISABLE_PHOTO)
    DISABLE_CAPTION = config["DEFAULT"].getboolean("disable_caption", DISABLE_CAPTION)

    # Email to telegram transfers
    TRANSFERS = []
    for section in config.sections():
        # Required variables
        chat_id = config[section]["chat_id"]

        # Optional variables
        to_address = config[section].get("to_address")
        from_address = config[section].get("from_address")
        disable_text = config[section].getboolean("disable_text", DISABLE_TEXT)
        disable_photo = config[section].getboolean("disable_photo", DISABLE_PHOTO)
        disable_caption = config[section].getboolean("disable_caption", DISABLE_CAPTION)
        caption_chat_id = config[section].get("caption_chat_id", None)

        TRANSFERS.append(
            {
                "chat_id": chat_id,
                "to_address": to_address,
                "from_address": from_address,
                "name": section,
                "disable_text": disable_text,
                "disable_photo": disable_photo,
                "disable_caption": disable_caption,
                "caption_chat_id": caption_chat_id,
            }
        )


for i in CONFIG_PATHS:
    path = pathlib.Path(i)
    if path.exists():
        load_config(path)
        break
else:
    logging.error("No valid 'config.ini' found")
    sys.exit()



import os
import logging
import mailbox
import pathlib
import mailparser


def get_emails(mail_path):
    mbox = mailbox.mbox(mail_path)
    parsed_emails = []
    try:
        mbox.lock()
    except mailbox.ExternalClashError:
        logging.error(
            "Mailbox not consumed, it is being access by another program"
        )
        return parsed_emails, 0

    keys = mbox.keys()
    for key in keys:
        email = mbox.pop(key)
        parsed_emails.append(mailparser.parse_from_string(str(email)))

    mbox.flush()
    mbox.unlock()
    mbox.close()
    return parsed_emails, 1


def get_attachments_paths(email, attachment_folder):
    email.write_attachments(attachment_folder)
    paths = []
    for attachment in email.attachments:
        paths.append(attachment_folder / pathlib.Path(attachment["filename"]))
    return paths

def clear_attachments(attachment_folder):
    try:
        for f in os.listdir(attachment_folder):
            os.remove(os.path.join(attachment_folder, f))
    except Exception as e:
        logging.error(e)

def should_skip_address(email, transfer, address):
    if transfer[address + "_address"] is None:
        logging.debug("%s : %s is None", transfer["name"], address)
        return False

    for check_address in email.mail[address]:
        if check_address[1] == transfer[address + "_address"]:
            logging.debug("%s : %s matches", transfer["name"], address)
            return False

    logging.debug(
        "%s : skipping since %s does not match", transfer["name"], address
    )
    return True



import logging

from time import time, sleep
from telegram import InputMediaPhoto
from telegram.ext import Updater, CommandHandler
from telegram.error import TelegramError


def api_interaction(func):
    def decorator(*args, **kwargs):
        self = args[0]
        if time() - self.last_message_time < self.message_timeout:
            logging.info("Sleeping for %s", self.message_timeout)
            sleep(self.message_timeout)

        try:
            func(*args, **kwargs)
        except TelegramError as error:
            logging.error(error)

        self.last_message_time = time()

    return decorator


class TelegramBot:
    def __init__(self, token, message_timeout):
        self.last_message_time = 0
        self.message_timeout = message_timeout
        self.updater = Updater(token=token, use_context=True)
        self.updater.dispatcher.add_handler(CommandHandler("ping", self._ping))
        self.updater.start_polling()

    def _ping(self, update, context):
        update.message.reply_text("pong")
        logging.debug("Replied to ping")

    @api_interaction
    def send_text(self, chat_id, text, disable_notification=False):
        self.updater.bot.sendMessage(
            chat_id=chat_id,
            text=text,
            disable_notification=disable_notification,
        )

    @api_interaction
    def send_photo(self, chat_id, paths, caption=None):
        length = len(paths)
        if length == 1:
            self.updater.bot.send_photo(
                chat_id=chat_id, photo=open(paths[0], "rb"), caption=caption
            )
        elif 2 <= length <= 10:
            media = []
            for path in paths:
                media.append(InputMediaPhoto(open(path, "rb")))

            media[0].caption = caption
            self.updater.bot.sendMediaGroup(chat_id=chat_id, media=media)
        else:
            logging.error(
                "paths length is too small or too big, length %s", str(length)
            )



import logging

from watchdog.events import FileSystemEventHandler

from .constants import MAIL_PATH, ATTACHMENTS_FOLDER, TRANSFERS
from .utils import get_emails, should_skip_address, get_attachments_paths


class EmailHandler(FileSystemEventHandler):
    def __init__(self, bot, lock_file, mail_path):
        self.bot = bot
        self.lock_file = lock_file
        self.mail_path = mail_path
        self.counter = 0
        logging.debug("Current count %s", self.counter)

    def consume_emails(self):
        emails, delete_count = get_emails(self.mail_path)
        self.counter += delete_count
        consume_emails(emails, self.bot)

    def on_deleted(self, event):
        logging.debug("Current count %s", self.counter)
        if event.src_path == self.lock_file:
            if self.counter <= 0:
                self.consume_emails()
                logging.debug("Current count %s", self.counter)
            else:
                self.counter -= 1
                logging.debug("Current count %s", self.counter)


def consume_emails(emails, bot):
    for email in emails:
        attachment_paths = get_attachments_paths(email, ATTACHMENTS_FOLDER)
        for transfer in TRANSFERS:
            # Skip if from_address or to_address matches or is None
            if should_skip_address(
                email, transfer, "to"
            ) or should_skip_address(email, transfer, "from"):
                continue

            # Send text or send photo if not disabled
            if len(attachment_paths) > 0 and not transfer["disable_photo"]:
                caption = email.subject + "\n" + "".join(email.text_plain)
                if transfer["caption_chat_id"]:
                    bot.send_text(
                        transfer["caption_chat_id"],
                        caption
                    )
                    caption = None
                elif transfer["disable_caption"]:
                    caption = None

                bot.send_photo(
                    transfer["chat_id"],
                    attachment_paths,
                    caption
                )
            elif not transfer["disable_text"]:
                bot.send_text(
                    transfer["chat_id"],
                    email.subject + "\n" + "".join(email.text_plain),
                )



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="email-to-telegram",
    version="1.1.1",
    description="Telegram bot that reads a mail file and forwards to Telegram chat",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ItsNotGoodName/email-to-telegram",
    author="ItsNotGoodName",
    author_email="gurnaindeol@gmail.com",
    packages=find_packages(exclude="test"),
    python_requires=">=3.6, <4",
    install_requires=["python-telegram-bot>=13.5,<14.0", "watchdog>=2.1.1,<3.0.0", "mail-parser>=3.15.0,<4.0.0"],
    entry_points={
        "console_scripts": [
            "email-to-telegram=email_to_telegram.__main__:main",
        ],
    },
    keywords="bot, telegram",
    project_urls={
        "Source": "https://github.com/ItsNotGoodName/email-to-telegram",
    },
)
